# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-05-29 12:46
from __future__ import unicode_literals

import django_extensions.db.fields.json
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('aa_stripe', '0017_stripecharge_manual_charge'),
    ]

    operations = [
        migrations.AddField(
            model_name='stripecustomer',
            name='sources',
            field=django_extensions.db.fields.json.JSONField(default=[]),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-06-06 09:36
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aa_stripe', '0018_stripecustomer_sources'),
    ]

    operations = [
        migrations.AddField(
            model_name='stripecustomer',
            name='default_source',
            field=models.CharField(blank=True, help_text='ID of default source from Stripe', max_length=255),
        ),
    ]



# -*- coding: utf-8 -*-
import logging

import stripe
from django.utils.translation import ugettext_lazy as _
from rest_framework import serializers
from rest_framework.exceptions import ValidationError
from rest_framework.serializers import JSONField, ModelSerializer

from aa_stripe.models import StripeCoupon, StripeCustomer, StripeWebhook

logging.getLogger("aa-stripe")


class StripeCouponSerializer(ModelSerializer):
    class Meta:
        model = StripeCoupon
        fields = [
            "coupon_id", "amount_off", "currency", "duration", "duration_in_months", "livemode", "max_redemptions",
            "metadata", "percent_off", "redeem_by", "times_redeemed", "valid", "is_created_at_stripe", "created",
            "updated", "is_deleted"
        ]


class StripeCustomerSerializer(ModelSerializer):
    stripe_js_response = JSONField()

    def create(self, validated_data):
        instance = None
        if validated_data.get("stripe_js_response"):
            # Create a Customer
            try:
                user = self.context['request'].user
                stripe_js_response = validated_data.pop("stripe_js_response")
                instance = StripeCustomer.objects.create(
                    user=user, stripe_js_response=stripe_js_response)
                instance.create_at_stripe()
            except stripe.error.StripeError as e:
                logging.error(
                    "[AA-Stripe] creating customer failed for user {user.id}: {error}".format(user=user, error=e)
                )
                raise ValidationError({"stripe_error": e._message})

        return instance

    class Meta:
        model = StripeCustomer
        fields = ["stripe_js_response"]


class StripeCustomerDetailsSerializer(ModelSerializer):
    # source data from Stripe JS response used to add a new card
    stripe_js_response = serializers.JSONField(write_only=True, required=True)
    sources = serializers.JSONField(read_only=True)

    def validate_stripe_js_response(self, value):
        if "id" not in value:
            raise serializers.ValidationError(_("This field must contain JSON data from Stripe JS."))
        return value

    def validate(self, data):
        if "stripe_js_response" not in data:
            raise ValidationError({"stripe_js_response": ["This field is required."]})
        return super(StripeCustomerDetailsSerializer, self).validate(data)

    def update(self, instance, validated_data):
        stripe_js_response = validated_data["stripe_js_response"]
        new_source_token = stripe_js_response["id"]
        try:
            instance.add_new_source(new_source_token, stripe_js_response)
        except stripe.error.StripeError as e:
            logging.error(
                "[AA-Stripe] adding new source to customer failed for user {user.id}: {error}".format(
                    user=self.context["request"].user, error=e)
            )
            raise ValidationError({"stripe_error": e._message})

        return instance

    class Meta:
        model = StripeCustomer
        fields = [
            "id", "user", "stripe_customer_id", "is_active", "sources", "default_source", "default_source_data",
            "stripe_js_response"
        ]
        read_only_fields = ["id", "user", "stripe_customer_id", "is_active", "sources", "default_source"]


class StripeWebhookSerializer(ModelSerializer):

    class Meta:
        model = StripeWebhook
        fields = ["id", "raw_data"]



# -*- coding: utf-8 -*-
__title__ = "Ro Stripe"
__version__ = "0.9.1"
__author__ = "Remigiusz Dymecki"
__license__ = "MIT"
__copyright__ = "Copyright 2019 Ro"

# Version synonym
VERSION = __version__



# -*- coding: utf-8 -*-
# Generated by Django 1.11.14 on 2018-09-13 20:48
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aa_stripe', '0019_stripecustomer_default_source'),
    ]

    operations = [
        migrations.AddField(
            model_name='stripecharge',
            name='statement_descriptor',
            field=models.CharField(blank=True, max_length=22),
        ),
    ]



# Generated by Django 2.1.11 on 2019-09-06 20:23

import django_extensions.db.fields.json
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aa_stripe', '0020_stripecharge_statement_descriptor'),
    ]

    operations = [
        migrations.AlterField(
            model_name='stripecharge',
            name='stripe_refund_id',
            field=models.CharField(blank=True, db_index=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='stripecharge',
            name='stripe_response',
            field=django_extensions.db.fields.json.JSONField(blank=True, default=dict),
        ),
        migrations.AlterField(
            model_name='stripecoupon',
            name='metadata',
            field=django_extensions.db.fields.json.JSONField(blank=True, default=dict, help_text='Set of key/value pairs that you can attach to an object. It can be useful for storing additional information about the object in a structured format.'),
        ),
        migrations.AlterField(
            model_name='stripecoupon',
            name='stripe_response',
            field=django_extensions.db.fields.json.JSONField(blank=True, default=dict),
        ),
        migrations.AlterField(
            model_name='stripecustomer',
            name='sources',
            field=django_extensions.db.fields.json.JSONField(blank=True, default=[]),
        ),
        migrations.AlterField(
            model_name='stripecustomer',
            name='stripe_js_response',
            field=django_extensions.db.fields.json.JSONField(blank=True, default=dict),
        ),
        migrations.AlterField(
            model_name='stripecustomer',
            name='stripe_response',
            field=django_extensions.db.fields.json.JSONField(blank=True, default=dict),
        ),
        migrations.AlterField(
            model_name='stripesubscription',
            name='metadata',
            field=django_extensions.db.fields.json.JSONField(blank=True, default=dict, help_text='https://stripe.com/docs/api/python#create_subscription-metadata'),
        ),
        migrations.AlterField(
            model_name='stripesubscription',
            name='stripe_response',
            field=django_extensions.db.fields.json.JSONField(blank=True, default=dict),
        ),
        migrations.AlterField(
            model_name='stripesubscriptionplan',
            name='metadata',
            field=django_extensions.db.fields.json.JSONField(blank=True, default=dict, help_text='A set of key/value pairs that you can attach to a plan object. It can be useful for storing additional information about the plan in a structured format.'),
        ),
        migrations.AlterField(
            model_name='stripesubscriptionplan',
            name='stripe_response',
            field=django_extensions.db.fields.json.JSONField(blank=True, default=dict),
        ),
        migrations.AlterField(
            model_name='stripewebhook',
            name='raw_data',
            field=django_extensions.db.fields.json.JSONField(blank=True, default=dict),
        ),
    ]



# -*- coding: utf-8 -*-
import sys

import stripe
from django.core.management.base import BaseCommand
from django.utils import timezone

from aa_stripe.models import StripeCustomer
from aa_stripe.settings import stripe_settings


class Command(BaseCommand):
    help = "Update customers card data from Stripe API"

    def handle(self, *args, **options):
        stripe.api_key = stripe_settings.API_KEY
        last_customer = None
        retry_count = 0
        updated_count = 0
        start_time = timezone.now()
        verbose = options["verbosity"] >= 2
        if verbose:
            print("Began refreshing customers")

        while True:
            try:
                response = stripe.Customer.list(limit=100, starting_after=last_customer)
            except stripe.error.StripeError:
                if retry_count > 5:
                    raise
                retry_count += 1
                continue
            else:
                retry_count = 0

            for stripe_customer in response["data"]:
                updated_count += StripeCustomer.objects.filter(stripe_customer_id=stripe_customer["id"]).update(
                    sources=stripe_customer["sources"]["data"], default_source=stripe_customer["default_source"]
                )

            if not response["has_more"]:
                break

            if verbose:
                sys.stdout.write(".")  # indicate that the command did not hung up
                sys.stdout.flush()
            last_customer = response["data"][-1]

        if verbose:
            if updated_count:
                print("\nCustomers updated: {} (took {:2f}s)".format(
                    updated_count, (timezone.now() - start_time).total_seconds()))
            else:
                print("No customers were updated.")
